package mfer

import (
	"fmt"
	"io"
	"io/fs"
	"path/filepath"
	"strings"

	"github.com/davecgh/go-spew/spew"
	"github.com/spf13/afero"
)

type ManifestFile struct {
	Path     string
	FileInfo fs.FileInfo
}

func (m *ManifestFile) String() string {
	return fmt.Sprintf("<File \"%s\">", m.Path)
}

type Manifest struct {
	SourceFS      afero.Fs
	SourceFSRoot  string
	Files         []*ManifestFile
	ScanOptions   *ManifestScanOptions
	TotalFileSize int64
}

type ManifestScanOptions struct {
	IgnoreDotfiles bool
	FollowSymLinks bool
}

func NewFromPath(inputPath string, options *ManifestScanOptions) (*Manifest, error) {
	abs, err := filepath.Abs(inputPath)
	if err != nil {
		return nil, err
	}
	afs := afero.NewBasePathFs(afero.NewOsFs(), abs)
	spew.Dump(afs)
	m, err := NewFromFS(afs, options)
	if err != nil {
		return nil, err
	}
	m.SourceFSRoot = abs
	spew.Dump(m)
	return m, nil
}

func NewFromFS(fs afero.Fs, options *ManifestScanOptions) (*Manifest, error) {
	m := &Manifest{
		SourceFS:    fs,
		ScanOptions: options,
	}
	m.Scan()
	return m, nil
}

func (m *Manifest) Scan() {
	afero.Walk(m.SourceFS, "./", func(path string, info fs.FileInfo, err error) error {

		if m.ScanOptions.IgnoreDotfiles && strings.HasPrefix(path, ".") {
			// FIXME make this check all path components BUG
			return nil
		}

		if info != nil && info.IsDir() {
			// manifests contain only files, directories are implied.
			return nil
		}

		fmt.Printf("path = %s\n", path)
		spew.Dump(path)
		spew.Dump(info)

		fileinfo, staterr := m.SourceFS.Stat(path)
		if staterr != nil {
			panic(staterr)
		}

		nf := &ManifestFile{
			Path:     path,
			FileInfo: fileinfo,
		}
		m.Files = append(m.Files, nf)
		m.TotalFileSize = m.TotalFileSize + info.Size()
		fmt.Printf("total file count now %i\n", len(m.Files))
		fmt.Printf("total file size now %i\n", m.TotalFileSize)
		return nil
	})
}

func (m *Manifest) Write(output io.Writer) error {
	return nil
}
